------------------------------------------------------------------------------
-- cmd_sm.vhd - entity/architecture pair
------------------------------------------------------------------------------
--
-- ***************************************************************************
-- ** Copyright (c) 1995-2006 Xilinx, Inc.  All rights reserved.            **
-- **                                                                       **
-- ** Xilinx, Inc.                                                          **
-- ** XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"         **
-- ** AS A COURTESY TO YOU, SOLELY FOR USE IN DEVELOPING PROGRAMS AND       **
-- ** SOLUTIONS FOR XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE,        **
-- ** OR INFORMATION AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE,        **
-- ** APPLICATION OR STANDARD, XILINX IS MAKING NO REPRESENTATION           **
-- ** THAT THIS IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,     **
-- ** AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE      **
-- ** FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY              **
-- ** WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE               **
-- ** IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR        **
-- ** REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF       **
-- ** INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS       **
-- ** FOR A PARTICULAR PURPOSE.                                             **
-- **                                                                       **
-- ***************************************************************************
--
------------------------------------------------------------------------------
-- Filename:          cmd_sm.vhd
-- Version:           1.00.c
-- VHDL Standard:     VHDL'93
------------------------------------------------------------------------------
------------------------------------------------------------------------------

-- DO NOT EDIT BELOW THIS LINE --------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

library proc_common_v2_00_a;
use proc_common_v2_00_a.proc_common_pkg.all;
-- DO NOT EDIT ABOVE THIS LINE --------------------

--USER libraries added here

------------------------------------------------------------------------------
-- Entity section
------------------------------------------------------------------------------
-- Definition of Generics:
--
-- Definition of Ports:
------------------------------------------------------------------------------

entity cmd_sm is
  generic
  (
    C_LCD_DWIDTH      : integer  := 4;
    C_CLK_PERIOD      : integer  := 10000;
    C_LCD_TC          : integer  := 500000;
    C_LCD_TW          : integer  := 230000;
    C_LCD_TSU1        : integer  := 40000;
    C_LCD_TH1         : integer  := 10000;
    C_LCD_TSU2        : integer  := 80000;
    C_LCD_TH2         : integer  := 10000;
    C_LCD_CMDDLY      : integer  := 49000
  );                                                          
  port
  (
    Clk               : in  std_logic;
    Rst               : in  std_logic;
    Wr_Req            : in  std_logic;
    Wr_Val            : in  std_logic_vector(7 downto 0);
    Req_Done          : out std_logic;
    Is4Bit            : in  std_logic;
    IsData            : in  std_logic;
	 LCD_E             : out std_logic;    
	 LCD_RW 	          : out std_logic;    
	 LCD_RS            : out std_logic;    
	 LCD_DB            : out std_logic_vector(C_LCD_DWIDTH-1 downto 0)
  );
end entity cmd_sm;

------------------------------------------------------------------------------
-- Architecture section
------------------------------------------------------------------------------

architecture IMP of cmd_sm is

-------------------------------------------------------------------------------
-- Function declarations
-------------------------------------------------------------------------------
-- Function end_cnt will set the end count value to zero if the count value
-- is zero, otherwise the end count value is set to the count value -2
-------------------------------------------------------------------------------

function end_cnt (clocks, bits : integer) return std_logic_vector is
variable end_vec : std_logic_vector(0 to bits -1);
begin
    if clocks > 2 then 
        end_vec := conv_std_logic_vector(clocks-2, bits);
    else
        end_vec := conv_std_logic_vector(1, bits);
    end if;
    
    return end_vec;
end function end_cnt;

-----------------------------------------------------------------------------
-- Constant declarations
-----------------------------------------------------------------------------
constant WRSU_CLKS      : integer := C_LCD_TSU1/C_CLK_PERIOD;
constant EMIN_CLKS      : integer := C_LCD_TW/C_CLK_PERIOD;
constant WRHD_CLKS      : integer := C_LCD_TH1/C_CLK_PERIOD;
constant WRCYC_CLKS     : integer := C_LCD_TC/C_CLK_PERIOD;
constant WRCMD_CLKS     : integer := C_LCD_CMDDLY/(C_CLK_PERIOD/1000);
constant C_CNT_MAX      : integer := max2(1,log2(WRCMD_CLKS));

constant C_WRSUCNT      : std_logic_vector(0 to C_CNT_MAX-1) := end_cnt(WRSU_CLKS,  C_CNT_MAX);
constant C_EMINCNT      : std_logic_vector(0 to C_CNT_MAX-1) := end_cnt(EMIN_CLKS,  C_CNT_MAX);
constant C_WRHDCNT      : std_logic_vector(0 to C_CNT_MAX-1) := end_cnt(WRHD_CLKS,  C_CNT_MAX);
constant C_WRCYCCNT     : std_logic_vector(0 to C_CNT_MAX-1) := end_cnt(WRCYC_CLKS, C_CNT_MAX);
constant C_WRCMDCNT     : std_logic_vector(0 to C_CNT_MAX-1) := end_cnt(WRCMD_CLKS, C_CNT_MAX);


  type CMD_STATE_TYPE is (IDLE, WRITE, WR_EN, WR_HD, WR_CY, WR_CYDLY, WR_CMDDLY);
  signal cmd_sm_cs        : CMD_STATE_TYPE;
  signal cmd_sm_ns        : CMD_STATE_TYPE;

  signal timer_cnt        : std_logic_vector(0 to C_CNT_MAX-1);
  signal timer_cnt_cmb    : std_logic_vector(0 to C_CNT_MAX-1);
  signal timer_load       : std_logic;
  signal timer_load_cmb   : std_logic;
  signal timer_end        : std_logic;

  signal write_done       : std_logic;
  signal write_done_cmb   : std_logic;
  signal do_two           : std_logic;
  signal do_two_cmb       : std_logic;

  signal E                : std_logic;    
  signal RW               : std_logic;    
  signal RS               : std_logic;    
  signal DB               : std_logic_vector(C_LCD_DWIDTH-1 downto 0);
  signal E_cmb            : std_logic;    
  signal RW_cmb           : std_logic;    
  signal RS_cmb           : std_logic;    
  signal DB_cmb           : std_logic_vector(C_LCD_DWIDTH-1 downto 0);

  signal debug            : std_logic;
  signal debug_cmb        : std_logic;

-----------------------------------------------------------------------------
-- Component declarations
-----------------------------------------------------------------------------
component mycounter
  generic ( 
        C_CLK_PERIOD     : integer := 10000;
        C_CNT_SIZE       : integer := 16
        );
  port (
       cnt_value         : in  std_logic_vector(0 to C_CNT_SIZE-1);
       cnt_load          : in  std_logic;
       cnt_end           : out std_logic;            
    
       Clk               : in  std_logic;
       Rst               : in  std_logic
       );
end component mycounter;

begin

COUNTER_1: mycounter
  generic map ( C_CLK_PERIOD => C_CLK_PERIOD,
                C_CNT_SIZE   => C_CNT_MAX
              )
  port map (    Clk       => Clk,
                Rst       => Rst,
                cnt_value => timer_cnt,
                cnt_load  => timer_load,
                cnt_end   => timer_end
           );

--------------------------------------------------------------------------------
-- Comand State Machine
-- CMD_SM_CMB:     combinational process for determining next state
-- CMD_SM_REG:     state machine registers
--------------------------------------------------------------------------------
    -- Combinational process
CMD_SM_CMB: process (cmd_sm_cs, Wr_Req, timer_end, timer_cnt, do_two, E, RW, RS, DB, debug, IsData, Is4Bit, Wr_Val)
begin

  -- Set default values
  cmd_sm_ns        <= cmd_sm_cs;
  timer_load_cmb   <= '0';
  timer_cnt_cmb    <= timer_cnt;
  write_done_cmb   <= '0';
  do_two_cmb       <= do_two;
  E_cmb            <= E;
  RW_cmb           <= RW;
  RS_cmb           <= RS;
  DB_cmb           <= DB;
debug_cmb        <= debug;

  case cmd_sm_cs is
------------------------- IDLE ----------------------------
    when IDLE =>
debug_cmb <= '0';
      if Wr_Req = '1' then
        cmd_sm_ns       <= WRITE;
        timer_load_cmb  <= '1';
        timer_cnt_cmb   <= C_WRSUCNT;
      end if;

------------------------- WRITE ---------------------------
    when WRITE =>
      if timer_end = '1' then
        cmd_sm_ns       <= WR_EN;
        RW_cmb          <= '0';
        RS_cmb          <= IsData;
        timer_load_cmb  <= '1';
        timer_cnt_cmb   <= C_WRSUCNT;
        if ( Is4Bit = '1' ) then
           do_two_cmb   <= '1';
           DB_cmb       <= Wr_Val(7 downto 4);
        else
           do_two_cmb   <= '0';
           DB_cmb      <= Wr_Val;
        end if;
      end if;

------------------------- WR_EN ---------------------------
    when WR_EN =>
      if timer_end = '1' then
		  E_cmb           <= '1';
        timer_load_cmb  <= '1';
        timer_cnt_cmb   <= C_EMINCNT;
        cmd_sm_ns       <= WR_HD;
      end if;

------------------------- WR_HD ---------------------------
    when WR_HD =>
      if timer_end = '1' then
		  E_cmb           <= '0';
        timer_load_cmb  <= '1';
        timer_cnt_cmb   <= C_WRHDCNT;
        cmd_sm_ns       <= WR_CY;
      end if;

------------------------- WR_CY ---------------------------
    when WR_CY =>
      if timer_end = '1' then
        timer_load_cmb   <= '1';
        timer_cnt_cmb    <= C_WRCYCCNT;

        if ( do_two = '1' ) then
          do_two_cmb     <= '0';
          DB_cmb         <= Wr_Val(3 downto 0);
          cmd_sm_ns      <= WR_EN;
        else
          cmd_sm_ns      <= WR_CYDLY;
        end if;
      end if;

------------------------- WR_CYDLY ---------------------------
    when WR_CYDLY =>
      if timer_end = '1' then
        timer_load_cmb  <= '1';
        timer_cnt_cmb   <= C_WRCMDCNT;
        cmd_sm_ns       <= WR_CMDDLY;
debug_cmb <= '1';
      end if;


------------------------- WR_CMDDLY ---------------------------
    when WR_CMDDLY =>
debug_cmb <= '1';
      if timer_end = '1' then
        write_done_cmb  <= '1';
        cmd_sm_ns       <= IDLE;
      end if;

-------------------------- DEFAULT ------------------------
    when others => 
        cmd_sm_ns <= IDLE;

end case;
end process CMD_SM_CMB;


  CMD_SM_REG: process (Clk)
  begin
    if (Clk'event and Clk = '1') then
      if (Rst = RESET_ACTIVE) then
        cmd_sm_cs    <= IDLE;
        timer_cnt    <= (others => '0');
        timer_load   <= '0';
        do_two       <= '0';
        write_done   <= '0';
        E            <= '0'; 
        RW           <= '0'; 
        RS           <= '0'; 
        DB           <= (others => '0'); 
        debug        <= '0';
      else
        cmd_sm_cs    <= cmd_sm_ns;
        timer_cnt    <= timer_cnt_cmb;
        timer_load   <= timer_load_cmb;
        do_two       <= do_two_cmb;
        write_done   <= write_done_cmb;
        E            <= E_cmb; 
        RW           <= RW_cmb; 
        RS           <= RS_cmb; 
        DB           <= DB_cmb; 
        debug        <= debug_cmb;
      end if;
    end if;
  end process CMD_SM_REG;


  Req_Done  <= write_done;
  LCD_E   <= E; 
  LCD_RW  <= RW; 
  LCD_RS  <= RS; 
  LCD_DB  <= DB; 

end IMP;
