--==================================================================================
-- Design unit  :  Crono Controller
--
-- File name    :  crono_ctrl.vhd
--                
-- Dependencies :  IEEE.Std_Logic_1164
--                 
--                
-- Author       :  MSeveri 
--                
-- Description  :  
--
-- Limitations  :  None
--
------------------------------------------------------------------------------------
-- Revision List
--
-- Version        Author       Date                  Changes
-- 1.0            MSev         06 February, 2008     First version
--==================================================================================


--==================================================================================
-- Library Definition
library ieee;
use ieee.std_logic_1164.all;
--==================================================================================


--==================================================================================
-- Entity Definition
--==================================================================================
entity crono_ctrl is
port
(
-- clock and reset -----------------------------------------------------------------
  arst       : IN std_logic;
  clk        : IN std_logic;
-- ctrl input ----------------------------------------------------------------------
  A          : IN std_logic;
  B          : IN std_logic;
-- ctrl output ---------------------------------------------------------------------
  time_incr   : OUT std_logic;
  time_rst    : OUT std_logic
);   
end entity crono_ctrl;
--==================================================================================

--==================================================================================
-- Architecture Description
--==================================================================================
architecture rtl of crono_ctrl is
------------------------------------------------------------------------------------
-- internal signals
------------------------------------------------------------------------------------
  type State is (idle, enable_counter, reset_counter);  
  signal state_c     : state;
  signal state_r     : state;
  
  signal time_incr_c : std_logic;
  signal time_rst_c  : std_logic;
------------------------------------------------------------------------------------

------------------------------------------------------------------------------------
begin

------------------------------------------------------------------------------------
-- State Machine Description - combinatorial logic
------------------------------------------------------------------------------------
 SMComb : process (state_r, A, b)
 begin 
 case state_r is
 
   when idle =>           
     if B='1' then 
       state_c <= reset_counter; 
     elsif (B='0' and A='1') then 
       state_c <= enable_counter; 
     else
       state_c     <= idle;
     end if;
     time_incr_c   <= '0';
     time_rst_c    <= '0';
       
   when enable_counter =>
     if A='1' then 
       state_c <= idle;
     else
       state_c <= enable_counter;
     end if;
     time_incr_c <= '1';
     time_rst_c  <= '0';

   when reset_counter =>
     if B='0' then
       state_c     <= idle;
       time_rst_c  <= '1';
     else
       state_c     <= idle;
       time_rst_c  <= '1';
     end if;
     time_incr_c   <= '0';
     
  end case;
 end process SMComb;
------------------------------------------------------------------------------------

------------------------------------------------------------------------------------
-- Infer Registers
------------------------------------------------------------------------------------
  SMSeq : process (arst, clk)
  begin
    if arst='1' then
      state_r   <= idle;
      time_incr <= '0';
      time_rst  <= '0';
    elsif clk'event and clk='1' then
      state_r   <= state_c;
      time_incr <= time_incr_c;
      time_rst  <= time_rst_c;
    end if;
  end process SMSeq;
------------------------------------------------------------------------------------
end rtl;
--==================================================================================