--==================================================================================
-- Design unit  :  Crono Controller
--
-- File name    :  crono_ctrl.vhd
--                
-- Dependencies :  IEEE.Std_Logic_1164
--                 
--                
-- Author       :  MSeveri 
--                
-- Description  :  
--
-- Limitations  :  None
--
------------------------------------------------------------------------------------
-- Revision List
--
-- Version        Author       Date                  Changes
-- 1.0            MSev         06 February, 2008     First version
--==================================================================================


--==================================================================================
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
--==================================================================================

--==================================================================================
-- Entity Description
--==================================================================================
entity debouncer is
port
(
 arst		   : IN std_logic;			-- asynchronous system reset
 clk		   : IN std_logic;			-- system clock
 d			   : IN std_logic;
 event_on	 : OUT std_logic;
 event_off : OUT std_logic
);
end debouncer;
--==================================================================================

--==================================================================================
-- Architecture Description
--==================================================================================
architecture rtl of debouncer is
------------------------------------------------------------------------------------
-- internal signal declaration
------------------------------------------------------------------------------------
signal d_s        : std_logic;
signal cnt        : unsigned(15 downto 0);
signal dly        : std_logic;
signal sat        : std_logic;
--------------------------------------------------------------------------------------
begin

--------------------------------------------------------------------------------------
-- Synchronize digital input
--------------------------------------------------------------------------------------
SynchonizeInput : process (arst, clk)
  variable v : std_logic_vector(1 downto 0);
begin
  if arst='1' then
    v:="00";
  elsif clk'event and clk='1' then
    v(1):=v(0); v(0):=d;
  end if;    
  d_s <= v(1);
end process SynchonizeInput;
-------------------------------------------------------------------

-------------------------------------------------------------------
-- Saturate counter before propagate input
-------------------------------------------------------------------
SaturateCounter : process (arst, clk)
begin
 if arst='1' then
   cnt       <= x"0000";
   event_on  <= '0';
   event_off <= '0';
   dly       <= '0';
  elsif clk'event and clk='1' then
    if d_s='0' and sat='0' then
       cnt <= x"0000";
    elsif d_s='0' and sat='1' then
       cnt <= x"0000";
    elsif d_s='1' and sat='0' then
       cnt <= cnt + 1;
    else
       cnt <= cnt;
    end if;
    dly       <= sat;
    event_on  <= (not dly) and sat;
    event_off <= (not sat) and dly;
  end if;
end process SaturateCounter;

sat <= '1' when cnt=x"ffff" else '0';
-------------------------------------------------------------------
  
--==================================================================
end rtl;
