------------------------------------------------------------------------------
-- init_sm.vhd - entity/architecture pair
------------------------------------------------------------------------------
--
-- ***************************************************************************
-- ** Copyright (c) 1995-2006 Xilinx, Inc.  All rights reserved.            **
-- **                                                                       **
-- ** Xilinx, Inc.                                                          **
-- ** XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"         **
-- ** AS A COURTESY TO YOU, SOLELY FOR USE IN DEVELOPING PROGRAMS AND       **
-- ** SOLUTIONS FOR XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE,        **
-- ** OR INFORMATION AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE,        **
-- ** APPLICATION OR STANDARD, XILINX IS MAKING NO REPRESENTATION           **
-- ** THAT THIS IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,     **
-- ** AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE      **
-- ** FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY              **
-- ** WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE               **
-- ** IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR        **
-- ** REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF       **
-- ** INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS       **
-- ** FOR A PARTICULAR PURPOSE.                                             **
-- **                                                                       **
-- ***************************************************************************
--
------------------------------------------------------------------------------
-- Filename:          init_sm.vhd
-- Version:           1.00.c
-- VHDL Standard:     VHDL'93
------------------------------------------------------------------------------
------------------------------------------------------------------------------

-- DO NOT EDIT BELOW THIS LINE --------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

library proc_common_v2_00_a;
use proc_common_v2_00_a.proc_common_pkg.all;
-- DO NOT EDIT ABOVE THIS LINE --------------------

--USER libraries added here

------------------------------------------------------------------------------
-- Entity section
------------------------------------------------------------------------------
-- Definition of Generics:
--
-- Definition of Ports:
------------------------------------------------------------------------------

entity init_sm is
  generic
  (
    C_LCD_DWIDTH      : integer  := 4;
    C_CLK_PERIOD      : integer  := 10000;
    C_LCD_PWRON       : integer  := 40000000;
    C_LCD_INITD1      : integer  := 4100000;
    C_LCD_INITD2      : integer  := 100000;
    C_LCD_INITD3      : integer  := 1500000;
    C_LCD_CMDDLY      : integer  := 49000;
    C_LCD_SETCMD      : integer  := 40;
    C_LCD_ONCMD       : integer  := 12;
    C_LCD_CLRCMD      : integer  := 1;
    C_LCD_MODECMD     : integer  := 6
  );                                                          
  port
  (
    Clk               : in  std_logic;
    Rst               : in  std_logic;
    Cmd_Done          : in std_logic;
    Cmd_Req           : out std_logic;
    Cmd_Val           : out std_logic_vector(0 to 7);
    Init_Done         : out std_logic
  );
end entity init_sm;

------------------------------------------------------------------------------
-- Architecture section
------------------------------------------------------------------------------

architecture IMP of init_sm is

-------------------------------------------------------------------------------
-- Function declarations
-------------------------------------------------------------------------------
-- Function end_cnt will set the end count value to zero if the count value
-- is zero, otherwise the end count value is set to the count value -2
-------------------------------------------------------------------------------

function end_cnt (clocks, bits : integer) return std_logic_vector is
variable end_vec : std_logic_vector(0 to bits -1);
begin
    if clocks > 2 then 
        end_vec := conv_std_logic_vector(clocks-2, bits);
    else
        end_vec := conv_std_logic_vector(1, bits);
    end if;
    
    return end_vec;
end function end_cnt;

-----------------------------------------------------------------------------
-- Constant declarations
-----------------------------------------------------------------------------
constant C_SETCMD       : std_logic_vector(0 to 7) := conv_std_logic_vector(C_LCD_SETCMD,  8);
constant C_ONCMD        : std_logic_vector(0 to 7) := conv_std_logic_vector(C_LCD_ONCMD,   8);
constant C_CLRCMD       : std_logic_vector(0 to 7) := conv_std_logic_vector(C_LCD_CLRCMD,  8);
constant C_MODECMD      : std_logic_vector(0 to 7) := conv_std_logic_vector(C_LCD_MODECMD, 8);

constant PWRUP_CLKS     : integer := C_LCD_PWRON/(C_CLK_PERIOD/1000);
constant DLY1_CLKS      : integer := C_LCD_INITD1/(C_CLK_PERIOD/1000);
constant DLY2_CLKS      : integer := C_LCD_INITD2/(C_CLK_PERIOD/1000);
constant DLY3_CLKS      : integer := C_LCD_INITD3/(C_CLK_PERIOD/1000);
constant CMD_CLKS       : integer := C_LCD_CMDDLY/(C_CLK_PERIOD/1000);
constant C_CNT_MAX     : integer := max2(1,log2(PWRUP_CLKS));

constant C_PWRUPCNT     : std_logic_vector(0 to C_CNT_MAX-1) := end_cnt(PWRUP_CLKS, C_CNT_MAX);
constant C_DLY1CNT      : std_logic_vector(0 to C_CNT_MAX-1) := end_cnt(DLY1_CLKS,  C_CNT_MAX);
constant C_DLY2CNT      : std_logic_vector(0 to C_CNT_MAX-1) := end_cnt(DLY2_CLKS,  C_CNT_MAX);
constant C_DLY3CNT      : std_logic_vector(0 to C_CNT_MAX-1) := end_cnt(DLY3_CLKS,  C_CNT_MAX);
constant C_CMDCNT       : std_logic_vector(0 to C_CNT_MAX-1) := end_cnt(CMD_CLKS,   C_CNT_MAX);


  type INIT_STATE_TYPE is (RESET, CMD1, CMD2, CMD3, CMD4, CMD5, CMD6,
                           DONE,  DLY1, DLY2, DLY3, DLY4, DLY5, DLY6 );
  signal init_sm_cs      : INIT_STATE_TYPE;
  signal init_sm_ns      : INIT_STATE_TYPE;

  signal initDone        : std_logic;
  signal initDone_cmb    : std_logic;
  signal init_req        : std_logic;
  signal init_req_cmb    : std_logic;
  signal init_val        : std_logic_vector(0 to 7);
  signal init_val_cmb    : std_logic_vector(0 to 7);

  signal timer_cnt       : std_logic_vector(0 to C_CNT_MAX-1);
  signal timer_cnt_cmb   : std_logic_vector(0 to C_CNT_MAX-1);
  signal timer_load      : std_logic;
  signal timer_load_cmb  : std_logic;
  signal timer_end       : std_logic;


-----------------------------------------------------------------------------
-- Component declarations
-----------------------------------------------------------------------------
component mycounter
  generic ( 
        C_CLK_PERIOD     : integer := 10000;
        C_CNT_SIZE       : integer := 16
        );
  port (
       cnt_value         : in  std_logic_vector(0 to C_CNT_SIZE-1);
       cnt_load          : in  std_logic;
       cnt_end           : out std_logic;            
    
       Clk               : in  std_logic;
       Rst               : in  std_logic
       );
end component mycounter;

begin

COUNTER_1: mycounter
  generic map ( C_CLK_PERIOD => C_CLK_PERIOD,
                C_CNT_SIZE   => C_CNT_MAX
              )
  port map (    Clk       => Clk,
                Rst       => Rst,
                cnt_value => timer_cnt,
                cnt_load  => timer_load,
                cnt_end   => timer_end
           );

--------------------------------------------------------------------------------
-- Initialization State Machine
-- INIT_SM_CMB:     combinational process for determining next state
-- INIT_SM_REG:     state machine registers
--------------------------------------------------------------------------------
    -- Combinational process
INIT_SM_CMB: process (init_sm_cs, timer_end, Cmd_Done, timer_cnt, initDone, init_val)
begin

  -- Set default values
  init_sm_ns       <= init_sm_cs;
  timer_load_cmb   <= '0';
  timer_cnt_cmb    <= timer_cnt;
  init_req_cmb     <= '0';
  initDone_cmb     <= initDone;
  init_val_cmb     <= init_val;

  case init_sm_cs is
------------------------- RESET ---------------------------
    when RESET =>
      initDone_cmb       <= '0';
      timer_load_cmb     <= '1';
      timer_cnt_cmb      <= C_PWRUPCNT;
      init_sm_ns         <= CMD1;

------------------------- CMD1 ----------------------------
    when CMD1 =>
      if timer_end = '1' then
        init_req_cmb     <= '1';
        init_val_cmb     <= C_SETCMD;
        init_sm_ns       <= DLY1;
      end if;

------------------------- DLY1 ----------------------------
    when DLY1 =>
      if Cmd_Done = '1' then
        timer_load_cmb   <= '1';
        timer_cnt_cmb    <= C_DLY1CNT;
        init_sm_ns       <= CMD2;
      end if;

------------------------- CMD2 ----------------------------
    when CMD2 =>
      if timer_end = '1' then
        init_req_cmb     <= '1';
        init_val_cmb     <= C_SETCMD;
        init_sm_ns       <= DLY2;
      end if;

------------------------- DLY2 ----------------------------
    when DLY2 =>
      if Cmd_Done = '1' then
        timer_load_cmb   <= '1';
        timer_cnt_cmb    <= C_DLY2CNT;
        init_sm_ns       <= CMD3;
      end if;

------------------------- CMD3 ----------------------------
    when CMD3 =>
      if timer_end = '1' then
        init_req_cmb     <= '1';
        init_val_cmb     <= C_SETCMD;
        init_sm_ns       <= DLY3;
      end if;

------------------------- DLY3 ----------------------------
    when DLY3 =>
      if Cmd_Done = '1' then
        timer_load_cmb   <= '1';
        timer_cnt_cmb    <= C_CMDCNT;
        init_sm_ns       <= CMD4;
      end if;

------------------------- CMD4 ----------------------------
    when CMD4 =>
      if timer_end = '1' then
        init_req_cmb     <= '1';
        init_val_cmb     <= C_ONCMD;
        init_sm_ns       <= DLY4;
      end if;

------------------------- DLY4 ----------------------------
    when DLY4 =>
      if Cmd_Done = '1' then
        timer_load_cmb   <= '1';
        timer_cnt_cmb    <= C_CMDCNT;
        init_sm_ns       <= CMD5;
      end if;

------------------------- CMD5 ----------------------------
    when CMD5 =>
      if timer_end = '1' then
        init_req_cmb     <= '1';
        init_val_cmb     <= C_CLRCMD;
        init_sm_ns       <= DLY5;
      end if;

------------------------- DLY5 ----------------------------
    when DLY5 =>
      if Cmd_Done = '1' then
        timer_load_cmb   <= '1';
        timer_cnt_cmb    <= C_DLY3CNT;
        init_sm_ns       <= CMD6;
      end if;

------------------------- CMD6 ----------------------------
    when CMD6 =>
      if timer_end = '1' then
        init_req_cmb     <= '1';
        init_val_cmb     <= C_MODECMD;
        init_sm_ns       <= DLY6;
      end if;

------------------------- DLY6 ----------------------------
    when DLY6 =>
      if Cmd_Done = '1' then
        timer_load_cmb   <= '1';
        timer_cnt_cmb    <= C_CMDCNT;
        init_sm_ns       <= DONE;
      end if;

---------------------------- DONE ---------------------------
    when DONE =>
      if timer_end = '1' then
        initDone_cmb  <= '1';
      end if;

-------------------------- DEFAULT --------------------------
    when others => 
        init_sm_ns <= RESET;

end case;
end process INIT_SM_CMB;


  INIT_SM_REG: process (Clk)
  begin
    if (Clk'event and Clk = '1') then
      if (Rst = RESET_ACTIVE) then
        init_sm_cs   <= RESET;
        initDone     <= '0';
        init_req     <= '0';
        init_val     <= (others => '0');
        timer_cnt    <= (others => '0');
        timer_load   <= '0';
      else
        init_sm_cs   <= init_sm_ns;
        initDone     <= initDone_cmb;
        init_req     <= init_req_cmb;
        init_val     <= init_val_cmb;
        timer_cnt    <= timer_cnt_cmb;
        timer_load   <= timer_load_cmb;
      end if;
    end if;
  end process INIT_SM_REG;


  Init_Done  <= initDone;
  Cmd_Req    <= init_req;
  Cmd_Val    <= init_val;

end IMP;
