--==================================================================================
-- Design unit  :  LCD Interface
--
-- File name    :  lcd_interface.vhd
--                
-- Dependencies :  IEEE.Std_Logic_1164
--                 
--                
-- Author       :  MSeveri 
--                
-- Description  :  
--
-- Limitations  :  None
--
------------------------------------------------------------------------------------
-- Revision List
--
-- Version        Author       Date                  Changes
-- 1.0            MSev         06 February, 2008     First version
--==================================================================================


--==================================================================================
-- Library Definition
library ieee;
use ieee.std_logic_1164.all;
--==================================================================================


--==================================================================================
-- Entity Definition
--==================================================================================
 entity LCD_Interface is
 port 
 (
 -- clock and reset ----------------------------------------------------------------
   arst     : IN std_logic;
   clk      : IN std_logic;
 -- ctrl input ---------------------------------------------------------------------
   hour     : IN std_logic_vector(7 downto 0); 
   minute   : IN std_logic_vector(7 downto 0);
   second   : IN std_logic_vector(7 downto 0);
   update   : IN std_logic;
 -- lcd interface signals ----------------------------------------------------------
   lcd_e    : OUT std_logic;
   lcd_rw   : OUT std_logic;    
   lcd_rs   : OUT std_logic;    
   lcd_db   : OUT std_logic_vector(7 downto 4)  -- use 4-bit interface
 );
 end entity LCD_Interface;
--==================================================================================

--==================================================================================
-- Architecture Description
--==================================================================================
architecture rtl of lcd_interface is
------------------------------------------------------------------------------------
-- internal signal
------------------------------------------------------------------------------------
signal lcd_if_reset : std_logic;
signal lcd_cmd_data : std_logic_vector(7 downto 0);
signal lcd_wrdata   : std_logic;
signal lcd_wrcmd    : std_logic;
------------------------------------------------------------------------------------
begin
------------------------------------------------------------------------------------
-- LCD Manager
------------------------------------------------------------------------------------
LCDManager : entity work.lcd_manager 
port map
(
 -- clock and reset ----------------------------------------------------------------
  clk => clk, arst => arst,
 -- ctrl input ---------------------------------------------------------------------
  hour => hour, minute => minute, second => second, update => update,
 -- control/data to lcd interface --------------------------------------------------
  lcd_if_reset => lcd_if_reset, lcd_cmd_data => lcd_cmd_data, 
  lcd_wrdata => lcd_wrdata, lcd_wrcmd  => lcd_wrcmd
); 
------------------------------------------------------------------------------------

------------------------------------------------------------------------------------
-- LCD Driver
------------------------------------------------------------------------------------
LCDDriver : entity work.LCD_Driver
generic map
(
-- use this for synthesis
  c_lcd_dwidth => 4, c_clk_period => 20000, c_lcd_tc => 500000,
  c_lcd_tw => 230000, c_lcd_tsu1 => 40000, c_lcd_th1 => 10000,
  c_lcd_tsu2 => 80000, c_lcd_th2 => 10000, c_lcd_pwron => 40000000,
  c_lcd_initd1 => 4100000, c_lcd_initd2 => 100000, 
  c_lcd_initd3 => 1500000, c_lcd_cmddly => 49000, c_lcd_setcmd => 40,
  c_lcd_oncmd => 12, c_lcd_clrcmd => 1, c_lcd_modecmd => 6,
  c_dwidth => 8, c_num_ce => 2
-- use this for simulazione
--  c_lcd_dwidth => 4, c_clk_period => 20000, c_lcd_tc => 500000,
--  c_lcd_tw => 230000, c_lcd_tsu1 => 40000, c_lcd_th1 => 10000,
--  c_lcd_tsu2 => 80000, c_lcd_th2 => 10000, c_lcd_pwron => 40000,
--  c_lcd_initd1 => 410000, c_lcd_initd2 => 100000, 
--  c_lcd_initd3 => 150000, c_lcd_cmddly => 49000, c_lcd_setcmd => 40,
--  c_lcd_oncmd => 12, c_lcd_clrcmd => 1, c_lcd_modecmd => 6,
--  c_dwidth => 8, c_num_ce => 2
)
port map
(
 -- LCD Interface signals ----------------------------------------------------------
  lcd_e => lcd_e, lcd_rw => lcd_rw, lcd_rs => lcd_rs, lcd_db => lcd_db,
 -- clock and reset ----------------------------------------------------------------
  bus2ip_clk => clk,  bus2ip_reset => lcd_if_reset,
 -- control/data interface ---------------------------------------------------------
  bus2ip_data => lcd_cmd_data, bus2ip_be => (others=>'1'), bus2ip_rdce => "00",
  bus2ip_wrce(1) => lcd_wrcmd, bus2ip_wrce(0) => lcd_wrdata,
  ip2bus_data => open, ip2bus_ack => open, ip2bus_retry => open,
  ip2bus_error => open, ip2bus_toutsup => open, ip2bus_postedwrinh => open
);
-------------------------------------------------------------------------------------

end rtl;
--==================================================================================