--==================================================================================
-- Design unit  :  Crono Example
--
-- File name    :  sim_crono_example.vhd
--                
-- Dependencies :  IEEE.Std_Logic_1164
--                 
--                
-- Author       :  MSeveri 
--                
-- Description  :  
--
-- Limitations  :  None
--
------------------------------------------------------------------------------------
-- Revision List
--
-- Version        Author       Date                  Changes
-- 1.0            MSev         06 February, 2008     First version
--==================================================================================


--==================================================================================
-- Library Definition
library ieee;
use ieee.std_logic_1164.all;
--==================================================================================


--==================================================================================
-- Entity Definition
--==================================================================================
entity sim_crono_example is
end entity sim_crono_example;
--==================================================================================


--==================================================================================
-- Architecture Description
--==================================================================================
architecture sim of sim_crono_example is
  signal arst     : std_logic;
  signal clk      : std_logic;
  signal button_A : std_logic;
  signal button_B : std_logic;
  signal lcd_e    : std_logic;
  signal lcd_rw   : std_logic;    
  signal lcd_rs   : std_logic;    
  signal lcd_db   : std_logic_vector(7 downto 0);
begin
------------------------------------------------------------------------------------
-- device under test
------------------------------------------------------------------------------------
dut : entity work.crono_example 
port map
(
-- clock and reset ------------------------------------------------------------------
  arst  => arst, clk => clk,
-- control input --------------------------------------------------------------------
  button_A => button_A,  button_B => button_B,
-- LCD interface --------------------------------------------------------------------
  lcd_e => lcd_e, lcd_rw => lcd_rw, lcd_rs => lcd_rs,
  lcd_db => lcd_db
);
------------------------------------------------------------------------------------

------------------------------------------------------------------------------------
-- Clock and reset
------------------------------------------------------------------------------------
 ClkProcess : process
 begin
   clk <= '0'; wait for 10 ns;
   clk <= '1'; wait for 10 ns;
 end process ClkProcess;

 RstProcess : process
 begin
   arst  <= '1'; wait for 10 us;
   arst  <= '0'; wait;
 end process RstProcess;
------------------------------------------------------------------------------------

------------------------------------------------------------------------------------
-- Stimuli
------------------------------------------------------------------------------------
 Stimuli : process
 begin
   button_A <= '0'; button_B <= '0';
   wait for 2 ms;
   wait until clk'event and clk='1'; button_A <= '1';
   wait for 10 ms;
   wait until clk'event and clk='1'; button_A <= '0';
   wait for 100 ms;
   wait until clk'event and clk='1'; button_A <= '1';
   wait for 10 ms;
   wait until clk'event and clk='1'; button_A <= '0';
   wait for 2 ms;
   wait until clk'event and clk='1'; button_B <= '1';
   wait for 10 ms;
   wait until clk'event and clk='1'; button_B <= '0';
   wait;
 end process Stimuli;

------------------------------------------------------------------------------------
end sim;
--==================================================================================