--==================================================================================
-- Design unit  :  Time Counter
--
-- File name    :  time_counter.vhd
--                
-- Dependencies :  IEEE.Std_Logic_1164
--                 
--                
-- Author       :  MSeveri 
--                
-- Description  :  
--
-- Limitations  :  None
--
------------------------------------------------------------------------------------
-- Revision List
--
-- Version        Author       Date                  Changes
-- 1.0            MSev         06 February, 2008     First version
--==================================================================================


--==================================================================================
-- Library Definition
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
--==================================================================================


--==================================================================================
-- Entity Definition
--==================================================================================
entity Time_Counter is
port 
(
-- clock and reset -----------------------------------------------------------------
  arst    : IN std_logic; -- asynchronous reset
  clk     : IN std_logic;
  reset   : IN std_logic; -- synchronous reset
-- ctrl input ----------------------------------------------------------------------
  ce      : IN std_logic;
  en      : IN std_logic;
-- time output ---------------------------------------------------------------------
  hour    : OUT std_logic_vector(7 downto 0);
  minute  : OUT std_logic_vector(7 downto 0);
  second  : OUT std_logic_vector(7 downto 0)
 );
end entity Time_Counter;
--==================================================================================

--==================================================================================
-- Architecture Description
--==================================================================================
architecture rtl of Time_Counter is
------------------------------------------------------------------------------------
-- internal signal
------------------------------------------------------------------------------------
signal hour_c    : std_logic_vector(7 downto 0);
signal minute_c  : std_logic_vector(7 downto 0);
signal second_c  : std_logic_vector(7 downto 0);

signal hour_r    : std_logic_vector(7 downto 0);
signal minute_r  : std_logic_vector(7 downto 0);
signal second_r  : std_logic_vector(7 downto 0);
------------------------------------------------------------------------------------
begin
------------------------------------------------------------------------------------
-- Time Counter for seconds
------------------------------------------------------------------------------------
SecCounterComb : process (second_r, en, reset)                   
  variable SecUnits_v : std_logic_vector(3 downto 0);
  variable SecTens_v  : std_logic_vector(3 downto 0);
begin
 SecUnits_v:=second_r(3 downto 0); SecTens_v:=second_r(7 downto 4);
 if en='1' then
  if SecUnits_v=x"9" then
    SecUnits_v:=x"0";
    if SecTens_v=x"5" then SecTens_v:=x"0"; 
    else SecTens_v:=SecTens_v+1;
    end if;
  else
    SecUnits_v:=SecUnits_v+1;
  end if;
 end if;
-- if reset='1' then
--  SecUnits_v:=x"0"; SecTens_v:=X"0";
-- end if;
 second_c <= SecTens_v & SecUnits_v;
end process SecCounterComb;
------------------------------------------------------------------------------------

------------------------------------------------------------------------------------
-- Time Counter for Minute
------------------------------------------------------------------------------------
MinuteCounterComb : process (second_r, minute_r, en, reset)                   
  variable MinUnits_v : std_logic_vector(3 downto 0);
  variable MinTens_v  : std_logic_vector(3 downto 0);
begin
 MinUnits_v:=minute_r(3 downto 0); MinTens_v:=minute_r(7 downto 4);
 if (second_r=x"59" and en='1') then
  if MinUnits_v=x"9" then
    MinUnits_v:=x"0";
    if MinTens_v=x"5" then MinTens_v:=x"0"; 
    else MinTens_v:=MinTens_v+1;
    end if;
  else
    MinUnits_v:=MinUnits_v+1;
  end if;
 end if;
-- if reset='1' then
--  MinUnits_v:=x"0"; MinTens_v:=x"0";
-- end if;
 minute_c <= MinTens_v & MinUnits_v;
end process MinuteCounterComb;
------------------------------------------------------------------------------------

------------------------------------------------------------------------------------
-- Time Counter for Hour
------------------------------------------------------------------------------------
HourCounterComb : process (second_r, minute_r, hour_r, en, reset)                   
  variable HourUnits_v : std_logic_vector(3 downto 0);
  variable HourTens_v  : std_logic_vector(3 downto 0);
begin
 HourUnits_v:=hour_r(3 downto 0); HourTens_v:=hour_r(7 downto 4);
 if (minute_r=x"59" and second_r=x"59" and en='1') then
  if Hour_r=x"24" then HourUnits_v:=x"0"; HourTens_v:=x"0";
  else
    if HourUnits_v=x"9" then
      HourUnits_v:=x"0"; HourTens_v:=HourTens_v+1;
    else
      HourUnits_v:=HourUnits_v+1;
   end if; 
  end if;
 end if;
-- if reset='1' then
--   HourUnits_v:=x"0"; HourTens_v:=x"0";
-- end if; 
 hour_c <= HourTens_v & HourUnits_v;
end process HourCounterComb;
------------------------------------------------------------------------------------

------------------------------------------------------------------------------------
-- Instantiare Register
------------------------------------------------------------------------------------
RegisterOutput : process (arst, clk)
begin
  if arst='1' then
    hour_r <= x"00"; minute_r  <= x"00"; second_r  <= x"00";    
  elsif clk'event and clk='1' then
    if reset='1' then
      hour_r <= x"00"; minute_r <= x"00"; second_r <= x"00";    
    elsif ce='1' then
      hour_r <= hour_c; minute_r <= minute_c; second_r <= second_c;    
    end if;
  end if;
end process RegisterOutput;
------------------------------------------------------------------------------------

------------------------------------------------------------------------------------
-- Drive Output
------------------------------------------------------------------------------------
  hour    <= hour_r;
  minute  <= minute_r;
  second  <= second_r;    
------------------------------------------------------------------------------------
end rtl;
--==================================================================================